use axum::{
  http::StatusCode,
  response::{IntoResponse, Response},
};

#[macro_export]
macro_rules! bail {
  ($msg:literal $(,)?) => {
    return Err(anyhow::anyhow!($msg).into())
  };
  ($err:expr $(,)?) => {
    return Err(anyhow::anyhow!($err).into())
  };
  ($fmt:expr, $($arg:tt)*) => {
    return Err(anyhow::anyhow!($fmt, $($arg)*).into())
  };
}

pub type Result<T, E = AppError> = std::result::Result<T, E>;

#[derive(Debug)]
pub struct AppError(anyhow::Error);

impl IntoResponse for AppError {
  fn into_response(self) -> Response {
    (
      StatusCode::INTERNAL_SERVER_ERROR,
      format!("Something went wrong: {}", self.0),
    )
      .into_response()
  }
}

impl<E> From<E> for AppError
where
  E: Into<anyhow::Error>,
{
  fn from(err: E) -> Self {
    Self(err.into())
  }
}
